import json
from datetime import datetime
class MyEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime):
            print("MyEncoder-datetime.datetime")
            return obj.strftime("%Y-%m-%d %H:%M:%S")
        if isinstance(obj, bytes):
            return str(obj, encoding='utf-8')
        if isinstance(obj, int):
            return int(obj)
        elif isinstance(obj, float):
            return float(obj)
        #elif isinstance(obj, array):
        #    return obj.tolist()
        else:
            return super(MyEncoder, self).default(obj)


import base64
import json
import logging
import os
import shlex
import subprocess
import sys
import time
from os.path import relpath
from pathlib import Path
from typing import Dict, List, Tuple
from urllib.parse import urlparse
import re
from os import path
import requests
import random
import shutil


from . import process_helper

logger = logging.getLogger(__name__)

# 源码入口脚本文件名。（按顺序搜索）
DEFAULT_ENTRY_SCRIPTS = "main entry deploy"
DEFAULT_GHTOKEN = os.environ.get('GHTOKEN')
GITCLONE_BASEDIR = path.join(os.getcwd(),".gitbase")
GITUP_LOGFILE=".gituplog.log"
def gitParseOwnerRepo(giturl:str):
    """再git url 中识别出owner 和 repo 名称"""
    parsed = urlparse(giturl)
    path = parsed.path
    m = re.match(r'/(.*)/(.*).git(.*)', path, flags=0)
    if m:
        owner = m.groups()[0]
        repo = m.groups()[1]
        file = m.groups()[2]
        return (owner,repo,file)
    return None

def gitup(giturl:str):
    """
        给定git 网址，下载到指定路径并根据规则运行相关代码。
    """
    parsed = urlparse(giturl)
    owner,repo,file = gitParseOwnerRepo(giturl)

    clone_to = path.join(GITCLONE_BASEDIR,repo)
    gitclone(owner,repo,parsed.username,clone_to)
    if not file:
        logger.info("no entry script,skip launch")
    if file:
        file = file.lstrip("/")
        scriptFile = path.join(clone_to,file)
        if not Path(scriptFile).exists():
            logger.warn(f"入口文件不存在{scriptFile}")

        logpath = path.join(clone_to,GITUP_LOGFILE)
        Path(scriptFile).chmod(0o700)
        logger.info(f"""========================================
[ ✅ gitup]
entry script\t: {scriptFile}
logs\t: tail -f {logpath} 
========================================
""")
        logfile = open(logpath,'w')
        subprocess.Popen([scriptFile],
            stdout=logfile, #输出重定向到文件。
            stderr=logfile,
            # stdout=subprocess.PIPE, 
            # stderr=subprocess.PIPE,
            cwd=clone_to,
            shell=False,
        )
    return logpath




def gitCopy(giturl:str, dest_dir:str):
    """
        以类似cp命令的方式，复制仓库中的文件到指定目录，
        为什么不直接用git clone? 因为如果目前文件夹非空，会运行失败        
    """
    parsed = urlparse(giturl)
    owner,repo,file = gitParseOwnerRepo(giturl)
    tmp_dir = "/tmp/gitclone_" + str(random.randint(0, 999999999))
    clone_to = path.join(tmp_dir,repo)
    gitclone(owner,repo,parsed.username,clone_to)
    logger.info(f"copytree from `{clone_to}` to `{dest_dir}` ")
    shutil.copytree(clone_to, dest_dir,dirs_exist_ok=True)

def dockerdev_git(giturl:str):
    """在容器内clone 源码，并启动为基于容器的开发环境"""
    parsed = urlparse(giturl)
    owner,repo,file = gitParseOwnerRepo(giturl)
    clone_to = path.join(repo)
    logger.info(f"clone 到 {clone_to}")
    gitclone(owner,repo,parsed.username,clone_to)
    

def gitclone(owner, repo, token, dest_dir):
    """克隆一个github仓库"""
    _token = token or os.environ.get("GITTOKEN")
        
    process_helper.exec(f"rm -rdf {dest_dir}")
    cmd = f"git clone --depth=1 https://{_token}@github.com/{owner}/{repo}.git {dest_dir}"
    logger.info(f"cmd {cmd}")
    cp = process_helper.exec(cmd, check=False)
    if cp.returncode != 0:
        logger.info(f"clone 失败 {owner}, {repo},{dest_dir}")
        logger.info(cp.stderr)
    else:
        logger.info("clone 成功")



def deploy_repo(repo_url):
    """  更新源码，目前配合github hook 的功能，暂时写死
        TODO: 目前仅支持ssh的方式拉取源码。要更新到支持github token"""
    # os.system('curl -q --insecure https://116.202.120.181/api/ip')
    # os.system('curl -q http://google.com')
    url = urlparse(repo_url)
    owner = url.path.split('/')[1]
    repo_name = url.path.split('/')[2].rstrip('.git')

    targetdir = "/deploy/" + owner + '/' + repo_name
    if os.path.exists(targetdir):
        logger.info("文件夹 %s 存在，拉取github" % targetdir)
        os.system("cd %s && git reset --hard origin/main && git pull" %
                  targetdir)
    else:
        parent_dir = Path(targetdir).parent
        Path(parent_dir).mkdir(mode=0o777, parents=True, exist_ok=True)
        logger.info("部署路径 %s" % targetdir)
        os.system("git clone {url} {targetdir}".format(url=repo_url,
                                                       targetdir=targetdir))
    # 运行
    is_entry_exists = False
    for entry_file in DEFAULT_ENTRY_SCRIPTS.split(' '):
        fullpath = os.path.join(targetdir, entry_file)
        logger.info("搜索入口脚本；{}".format(fullpath))
        if os.path.exists(fullpath):
            is_entry_exists = True
            logger.info("执行脚本 %s " % targetdir)
            proc = subprocess.Popen(['sh', '-c', fullpath], cwd=targetdir)
            break
    if not is_entry_exists:
        logger.warn("仓库入库脚本文件'{}'不存在，跳过启动".format(DEFAULT_ENTRY_SCRIPTS))


# class GHRepo_old():
#     """这是旧的，使用的第三方库，感觉累赘。"""
#     def __init__(self, token, owner, repo):
#         self.token = token
#         self.owner = owner
#         self.g = Github(token)
#         self.repo = self.g.get_repo(
#             "{owner}/{repo}".format(owner=owner, repo=repo))

#     def gh_writefile(self, path, content_text):
#         """在仓库对应路径写文件"""
#         # repo = self.g.get_repo("{owner}/{repo}".format(owner=self.owner, repo=self.repo))
#         try:
#             content = self.repo.get_contents(path)
#             # 更新
#             self.repo.update_file(content.path, "modi",
#                                   content_text, content.sha)
#         except UnknownObjectException as e:
#             # 文件不存在,创建
#             self.repo.create_file(path, "create", content_text)

#     def gh_readfile(self, path):
#         """在仓库对应路径读文件"""
#         repo = self.g.get_repo(
#             "{owner}/{repo}".format(owner=self.owner, repo=self.repo))
#         content = repo.get_contents(path)
#         context_bytes = base64.b64decode(content.content)
#         context_text = context_bytes.decode()
#         print(context_text)

#     def putFiles(self, local_dir):
#         """将文件夹内的所有文件上传(更新)到仓库上"""
#         dir = os.path.abspath(local_dir)
#         for home, dirs, files in os.walk(dir):
#             # print(home, dirs, files)
#             for file in files:
#                 relPath = str(Path(home) / file)[len(dir) + 1:]
#                 # print("相对路径: %s" % relPath)
#                 with open(str(Path(home) / file)) as f:
#                     workflow_text = f.read()
#                     self.gh_writefile(relPath, workflow_text)

#     def workflow_dispatch(self, name):
#         """手动调度任务"""
#         workflows = [x for x in self.repo.get_workflows()]
#         print(workflows)
#         for w in workflows:
#             if w.name == name:
#                 print("调度工作流 {}".format(name))
#                 if w.create_dispatch(ref="main"):
#                     print("调度成功")

#     def workflows(self):
#         """列出所有工作流"""
#         workflows = [x for x in self.repo.get_workflows()]
#         return workflows


def file_base64(data):
    data_b64 = base64.b64encode(data).decode('utf-8')
    return data_b64


class GHRepo():
    """对一个github repo进行操作"""

    def __init__(self,
                 url: str = None,
                 token: str = None,
                 owner: str = None,
                 repo: str = None):
        if url:
            repourl = urlparse(url)
            self.token = repourl.password
            self.owner = repourl.username
            pathitems = repourl.path.split('/')
            self.repo = pathitems[1]
            print('REPO')
            print(
                f'根据网址否则ghrepo， token {self.token}, owner:{self.owner}, repo:{self.repo}'
            )
        else:
            self.token = token
            self.owner = owner
            self.repo = repo

        self.http_headers = {
            'Authorization':
            f'token {self.token}',
            'Accept':
            'application/vnd.github.v3+json',
            'User-Agent':
            'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/63.0.3239.132 Safari/537.36 QIHU 360SE'
        }

    def repoInfo(self):
        """获取当前仓库的信息"""
        r = requests.get(f'https://api.github.com/orgs/{self.owner}/repos',
                         data=payload,
                         headers=self.http_headers)
        # text = r.text
        print("DEBUG 当前仓库信息", json.loads(r.text))

    def _put(apiurl, payload):
        """发出put请求"""
        return requests.put(f'https://api.github.com/orgs/{self.owner}/repos',
                            data=payload,
                            headers=self.http_headers)

    # 将文件转换为base64编码，上传文件必须将文件以base64格式上传

    def delete_file(self, file_path, sha, message="api delete"):
        url = f"https://api.github.com/repos/{self.owner}/{self.repo}/contents/{file_path}"
        headers = {"Authorization": "token " + self.token}
        payload = json.dumps({
            "sha": sha,
            "message": message,
        })
        resp = requests.delete(url=url, data=payload, headers=headers)
        json_data = resp.json()
        # print("删除结果：")
        # print(json_data)
        message = json_data.get("message")
        if message == "Not Found":
            return None
        return json_data

    def get_file_content(self, file_path: str):
        url = f"https://api.github.com/repos/{self.owner}/{self.repo}/contents/{file_path}"
        headers = {"Authorization": "token " + self.token}
        resp = requests.get(url=url, data=None, headers=headers)
        return resp.json()
        # print(json_data)
        # message = json_data.get("message")
        # if message == "Not Found":
        #     return None
        # return json_data

    def _write_content(self, file_path: str, file_data: bytes):
        req = requests.put(
            url=
            f"https://api.github.com/repos/{self.owner}/{self.repo}/contents/{file_path}",
            data=json.dumps({
                "message": "message",
                "committer": {
                    "name": "[user]",
                    "email": "user@163.com"
                },
                "content": file_base64(file_data)
            }),
            headers={"Authorization": "token " + self.token})
        req.encoding = "utf-8"
        return json.loads(req.text)

    # 上传文件（单个）
    def write_file_content(self,
                           file_path: str,
                           file_data: bytes,
                           skipExists=False):
        """以覆盖的方式写入文件到github仓库"""
        contentJson = self.get_file_content(file_path)
        sha = contentJson.get("sha")
        if sha and not skipExists:
            print(f"删除旧文件：{file_path}")
            self.delete_file(file_path, sha=sha)
            self._write_content(file_path, file_data)
        elif sha and skipExists:
            print(f"跳过{file_path}")
        else:
            self._write_content(file_path, file_data)

    def write_file_content_mutil(self, files=[]):
        """以覆盖的方式写入文件到github仓库"""
        for file in files:
            print(f"上传：{file}")
            self.write_file_content(file["path"], file["content"])

        # print(json.dumps(re_data))
        # print(f"https://cdn.jsdelivr.net/gh/{self.owner}/{self.repo}/{file_path}")



#!/usr/bin/env python3
import sys
import os
from mtlibs.docker_helper import isInContainer
from mtlibs import process_helper
from pathlib import Path
import logging
from dotenv import load_dotenv, find_dotenv
import json
import logging
import shlex
import subprocess
import time
from os.path import relpath
from typing import Dict, List, Tuple
from urllib.parse import urlparse
import re
from os import path
import argparse
from mtlibs.github import gitclone,gitParseOwnerRepo
from .. service.phpfpm import start_phpfpm
ENV_FILE = find_dotenv()
if ENV_FILE:
    load_dotenv(ENV_FILE)

load_dotenv(".env")

import logging
logging.basicConfig(level = logging.DEBUG,format = '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)

def main():
    logger.info(f"启动phpfpm7.4")
    start_phpfpm()

if __name__ == '__main__':
    main()




# auth0 认证实用工具
#

import email
import json
import os

from django.contrib.auth import authenticate
import jwt
import requests
from django.conf import settings
from jwt import PyJWKClient
from django.contrib.auth.models import User
from rest_framework import authentication
from rest_framework import exceptions


def get_token_auth_header(request):
    """从http header 总获取access_token"""
    auth = request.META.get("HTTP_AUTHORIZATION", None)
    if auth:
        parts = auth.split()
        token = parts[1]
        return token
    return None

# def jwt_get_username_from_payload_handler(payload):
#     """
#         提示：payload中，没有直接包含详细的用户信息，但是sub字段中包含了用户ID。
#               代码是来自于官方范例。
#     """
#     username = payload.get('sub').replace('|', '.')
#     authenticate(remote_user=username)
#     return username


# def jwt_decode_token(token):
#     header = jwt.get_unverified_header(token)
#     auth0_domain = settings.AUTH0_DOMAIN  # os.environ.get('AUTH0_DOMAIN')
#     jwks = requests.get(
#         'https://{}/.well-known/jwks.json'.format(auth0_domain)).json()
#     public_key = None
#     for jwk in jwks['keys']:
#         if jwk['kid'] == header['kid']:
#             public_key = jwt.algorithms.RSAAlgorithm.from_jwk(json.dumps(jwk))

#     if public_key is None:
#         raise Exception('Public key not found.')
#     api_identifier = settings.API_IDENTIFIER

#     issuer = 'https://{}/'.format(auth0_domain)
#     return jwt.decode(token, public_key, audience=api_identifier, issuer=issuer, algorithms=['RS256'])


def jwt_decode_token(token):
    """这个函数同jwt_decode_token， 只是不同的写法，留作参考。"""
    # url = f'https://{settings.AUTH0_DOMAIN}/.well-known/jwks.json'
    url = f'https://cognito-idp.us-east-1.amazonaws.com/us-east-1_cT1Z4oABW/.well-known/jwks.json'
    client_id = '7e7m592m7oceudmca3i12b1vmi'
    audience=client_id,
    jwks_client = PyJWKClient(url)
    signing_key = jwks_client.get_signing_key_from_jwt(token)
    return jwt.decode(
        token,
        signing_key.key,
        algorithms=['RS256'],
        audience=audience
    )


#!/usr/bin/env python3
import os
import sys
import subprocess
import shlex
from dotenv import load_dotenv, find_dotenv
import logging
logging.basicConfig(level = logging.DEBUG,format = '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)

ENV_FILE = find_dotenv()
if ENV_FILE:
    load_dotenv(ENV_FILE)

load_dotenv(".env")


def main():
    items = {k: os.environ.get(k) for k in os.environ.keys()}
    docker_compose_addi_args = items.get("DOCKER_COMPOSE_ARGS","")
    argv = " ".join(sys.argv[1:])
    cmd = f"docker-compose {docker_compose_addi_args} {argv}"
    logger.info(f"执行命令：{cmd}")
    result2 = subprocess.run(shlex.split(cmd))
    if result2.returncode != 0:
        logger.error(f"exec with error {result2}")

if __name__ == "__main__":
    main()



import sys
import os
from mtlibs.docker_helper import isInContainer
from mtlibs import process_helper
from pathlib import Path
import logging
from os import path
from urllib.parse import urlparse
from dotenv import load_dotenv, find_dotenv
from mtlibs.github import gitup,gitParseOwnerRepo,gitclone
ENV_FILE = find_dotenv()
if ENV_FILE:
    load_dotenv(ENV_FILE)

logging.basicConfig(level = logging.DEBUG,format = '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)

def start_sshd():
    public_key = """ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABgQDBYOFBjYyNpU/2gTpySjQz5WMMN9hfHRxQOAy1hPIGVpDncutkN67zXP2tBILbrxRNvltN3uO2fv3MV4scBDwV4z5AJfc9urlIvHrDfUYwhQVjqN/zza4aYKmIGi5b1B0Lve4/VSDqfjAaAhrPliWxx4mYUrLZREDwVyQocq6zuvUXkknNqr9/7TxCwNs37k7tSXPIpe+rbO2b+aADM3voq9ta30qaqO76HfA/R+YnfRtWth5uD+dYZy85mJPWD6Z6kOo4Uwb86pn8uf/yQ1ZyCDPp0P72feMKJGVgInM0jw+j44qlfOfXgq3MnjpBLEP7lZ2tC4MzMinvEdUpcNNxEjHNwsIKRxcAaiB0COc0rNNJLWoJndHO0IAqp1ZKjbRb+/Pco8WxL3Updkk23WZZgVyUtvI+wrmrDT4s73x/C28lQkWC+YLuKJOLKYAG2rvXaZifO1pFJE6pcvrP1ciXvjIG6YKIU6m9FrHZTz7CXgpy42p0SM8jqYODGfPVaEE= jamal@fabrikam.com
"""
    logger.info("设置并启动ssh服务")
    public_dir = os.path.join(os.path.expanduser('~'),".ssh/authorized_keys")
    if not Path(public_dir).exists():
        Path(public_dir).parent.mkdir(mode=0o600, exist_ok=True)
        Path(public_dir).touch(mode=0o600)
    
    
    with open(public_dir,'w') as f:
        f.write(public_key)
    process_helper.exec("service ssh start")

def setup_git():
    logger.info("设置git环境")
    os.system(f"git config --global user.email a@a.com")
    os.system(f"git config --global user.name a")
    
def setup_dockcer_env():
    logger.info("登录docker hub")
    DOCKER_HUB_USER=os.environ.get("DOCKER_HUB_USER")
    DOCKER_HUB_PASSWORD=os.environ.get("DOCKER_HUB_PASSWORD")
    os.system(f"""echo {DOCKER_HUB_PASSWORD} | docker login --username {DOCKER_HUB_USER} --password-stdin""")

def setup_gateway():
    """如果设置了代理网关，这将容器的默认网关ip设置为这个网关IP"""
    gateway_ip = os.environ.get("MTX_GATEWAY_IP")
    if gateway_ip:
        logger.info(f"网关IP: {gateway_ip}")
        os.system(f"ip route del default")
        os.system(f"route add default gw {gateway_ip}")
        
        logger.info("测试外网IP")
        os.system("curl --insecure ipinfo.io")
        logger.info(f"=====================================\n请查看上方的IP输出，看是否正确!!!, 如果出现dns问题，\n例如：无法解释域名等错误，请正确设置容器的DNS为: {gateway_ip}\n======================================\n")
    else:
        logger.info("没有指定代理网关，跳过网关设置")
        
def main():
    setup_gateway()
    giturl = os.environ.get("MTX_DEV_GIT")
    if not giturl:
        logger.info("no git url , skip clone")
    else:
        logger.info(f"gitup env: {giturl}")
        items = giturl.split("|")
        for item in items:
            logger.info(f"item: {item}")
            parsed = urlparse(giturl)
            owner,repo,file = gitParseOwnerRepo(giturl)
            clone_to = path.join(os.getcwd(),repo)
            logger.info(f"clone 到 {clone_to}")
            if Path(clone_to).exists():
                logger.info("target dir exists , skip clone")
            else:
                gitclone(owner,repo,parsed.username,clone_to)
                logger.info("clone finish")
   
    start_sshd()
    setup_git()
    logger.info("ready!")
    process_helper.exec("sleep infinity")
        
if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import sys
import os
from os.path import join
from pathlib import Path
import subprocess
import time
import subprocess
from os import path
import threading
import yaml
import logging
import shutil
from mtlibs import process_helper
from mtlibs import mtutils
import platform

logger = logging.getLogger(__name__)


class PhpFpmService():
    def __init__(self):
        pass
        # self.html_root = settings.getHtmlRoot()
        # logger.info(f"html root {self.html_root}")
        # self.smapiRoot = "/smapi"

    def start(self):
        logger.info("启动phpfpm服务进程")
        curr_php_fpm_conf = os.join(os.getcwd(), "php-fpm.conf")
        if Path(curr_php_fpm_conf).exists():
            logger.info(f"复制php-fpm配置文件到 /etc/php/7.4/fpm/php-fpm.conf")
            shutil.copy(curr_php_fpm_conf, "/etc/php/7.4/fpm/php-fpm.conf")
        port = 9000
        check_port = mtutils.get_tcp_open_port(9000)
        if not check_port:
            logger.info(f"tcp port opend {port},skip start phpFpm service")
        else:
            start_phpfpm()

    def stop(self):
        pass


def start_phpfpm(port=9000):
    fpmconf = """;;;;;;;;;;;;;;;;;;;;;
  
; FPM Configuration ;
;;;;;;;;;;;;;;;;;;;;;
; /etc/init.d/php7.4-fpm restart
; /etc/php/7.4/fpm/php-fpm.conf
; All relative paths in this configuration file are relative to PHP's install
; prefix (/usr). This prefix can be dynamically changed by using the
; '-p' argument from the command line.

;;;;;;;;;;;;;;;;;;
; Global Options ;
;;;;;;;;;;;;;;;;;;

[global]
; Pid file
; Note: the default prefix is /var
; Default Value: none
; Warning: if you change the value here, you need to modify systemd
; service PIDFile= setting to match the value here.
pid = /run/php7.4-fpm.pid

; Error log file
; If it's set to "syslog", log is sent to syslogd instead of being written
; into a local file.
; Note: the default prefix is /var
; Default Value: log/php-fpm.log
error_log = /var/log/php7.4-fpm.log

; syslog_facility is used to specify what type of program is logging the
; message. This lets syslogd specify that messages from different facilities
; will be handled differently.
; See syslog(3) for possible values (ex daemon equiv LOG_DAEMON)
; Default Value: daemon
;syslog.facility = daemon

; syslog_ident is prepended to every message. If you have multiple FPM
; instances running on the same server, you can change the default value
; which must suit common needs.
; Default Value: php-fpm
;syslog.ident = php-fpm

; Log level
; Possible Values: alert, error, warning, notice, debug
; Default Value: notice
;log_level = notice

; Log limit on number of characters in the single line (log entry). If the
; line is over the limit, it is wrapped on multiple lines. The limit is for
; all logged characters including message prefix and suffix if present. However
; the new line character does not count into it as it is present only when
; logging to a file descriptor. It means the new line character is not present
; when logging to syslog.
; Default Value: 1024
;log_limit = 4096

; Log buffering specifies if the log line is buffered which means that the
; line is written in a single write operation. If the value is false, then the
; data is written directly into the file descriptor. It is an experimental
; option that can potentionaly improve logging performance and memory usage
; for some heavy logging scenarios. This option is ignored if logging to syslog
; as it has to be always buffered.
; Default value: yes
;log_buffering = no

; If this number of child processes exit with SIGSEGV or SIGBUS within the time
; interval set by emergency_restart_interval then FPM will restart. A value
; of '0' means 'Off'.
; Default Value: 0
;emergency_restart_threshold = 0

; Interval of time used by emergency_restart_interval to determine when
; a graceful restart will be initiated. This can be useful to work around
; accidental corruptions in an accelerator's shared memory.
; Available Units: s(econds), m(inutes), h(ours), or d(ays)
; Default Unit: seconds
; Default Value: 0
;emergency_restart_interval = 0

; Time limit for child processes to wait for a reaction on signals from master.
; Available units: s(econds), m(inutes), h(ours), or d(ays)
; Default Unit: seconds
; Default Value: 0
;process_control_timeout = 0

; The maximum number of processes FPM will fork. This has been designed to control
; the global number of processes when using dynamic PM within a lot of pools.
; Use it with caution.
; Note: A value of 0 indicates no limit
; Default Value: 0
; process.max = 128

; Specify the nice(2) priority to apply to the master process (only if set)
; The value can vary from -19 (highest priority) to 20 (lowest priority)
; Note: - It will only work if the FPM master process is launched as root
; - The pool process will inherit the master process priority
; unless specified otherwise
; Default Value: no set
; process.priority = -19

; Send FPM to background. Set to 'no' to keep FPM in foreground for debugging.
; Default Value: yes
;daemonize = yes

; Set open file descriptor rlimit for the master process.
; Default Value: system defined value
;rlimit_files = 1024

; Set max core size rlimit for the master process.
; Possible Values: 'unlimited' or an integer greater or equal to 0
; Default Value: system defined value
;rlimit_core = 0

; Specify the event mechanism FPM will use. The following is available:
; - select (any POSIX os)
; - poll (any POSIX os)
; - epoll (linux >= 2.5.44)
; - kqueue (FreeBSD >= 4.1, OpenBSD >= 2.9, NetBSD >= 2.0)
; - /dev/poll (Solaris >= 7)
; - port (Solaris >= 10)
; Default Value: not set (auto detection)
;events.mechanism = epoll

; When FPM is built with systemd integration, specify the interval,
; in seconds, between health report notification to systemd.
; Set to 0 to disable.
; Available Units: s(econds), m(inutes), h(ours)
; Default Unit: seconds
; Default value: 10
;systemd_interval = 10

;;;;;;;;;;;;;;;;;;;;
; Pool Definitions ;
;;;;;;;;;;;;;;;;;;;;

; Multiple pools of child processes may be started with different listening
; ports and different management options. The name of the pool will be
; used in logs and stats. There is no limitation on the number of pools which
; FPM can handle. Your system will tell you anyway :)

; Include one or more files. If glob(3) exists, it is used to include a bunch of
; files from a glob(3) pattern. This directive can be used everywhere in the
; file.
; Relative path can also be used. They will be prefixed by:
; - the global prefix if it's been set (-p argument)
; - /usr otherwise
include=/etc/php/7.4/fpm/pool.d/*.conf


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; custom begin
;;;;;;;;;;;;;;
listen = 127.0.0.1:@@PORT@@
; Clear environment in FPM workers
; Prevents arbitrary environment variables from reaching FPM worker processes
; by clearing the environment in workers before env vars specified in this
; pool configuration are added.
; Setting to "no" will make all environment variables available to PHP code
; via getenv(), $_ENV and $_SERVER.
; Default Value: yes
clear_env = no
user = nginx
"""
# 注意（上方配置）当前用户 user = nginx，跟nginx配置的用户一致。
    with open("/etc/php/7.4/fpm/php-fpm.conf", "w") as f:
        f.write(fpmconf.replace("@@PORT@@", str(port)))

    phpfpm_cp = process_helper.exec("/etc/init.d/php7.4-fpm restart")
    if phpfpm_cp.returncode == 0:
        logger.info("phpfpm 启动成功")
    else:
        logger.info("phpfpm 启动失败")



import asyncio
import os
from time import time
from git.repo import Repo
from pathlib import Path
from csservices import portforward
import time
from mtlibs import mtutils

def on_command(args):
    """启动ssh服务器"""
    print('启动ssh服务器')
    mtutils.init_sshd()
    os.system("sudo service ssh start")
    while True:
        time.sleep(1)



# 全新创建rancher服务资源组。

# bash 获取登陆凭证：
# az ad sp create-for-rbac --name localtest-sp-rbac --skip-assignment


import os
import paramiko
from azure.mgmt.resource import ResourceManagementClient
from azure.identity import AzureCliCredential
from azure.identity import AzureCliCredential
from azure.mgmt.resource import ResourceManagementClient
from azure.mgmt.network import NetworkManagementClient
from azure.mgmt.compute import ComputeManagementClient


os.environ["AZURE_SUBSCRIPTION_ID"] = "f6038766-7885-4773-974b-be0383525457"
os.environ["AZURE_LOCATION"] = "southeastasia"
# Acquire a credential object using CLI-based authentication.
credential = AzureCliCredential()
# Retrieve subscription ID from environment variable.
subscription_id = os.environ["AZURE_SUBSCRIPTION_ID"]

def create_vm_rancher(location="southeastasia"):
    print("创建rancher 虚拟机")
    credential = AzureCliCredential()
    subscription_id = os.environ["AZURE_SUBSCRIPTION_ID"]
    resource_client = ResourceManagementClient(credential, subscription_id)
    RESOURCE_GROUP_NAME = "ranchar-server-4"
    rg_result = resource_client.resource_groups.create_or_update(RESOURCE_GROUP_NAME,
        {
            "location": location
        }
    )
    print(f"Provisioned resource group {rg_result.name} in the {rg_result.location} region")
    VNET_NAME = "python-example-vnet"
    SUBNET_NAME = "python-example-subnet"
    IP_NAME = "python-example-ip"
    IP_CONFIG_NAME = "python-example-ip-config"
    NIC_NAME = "python-example-nic"
    # Obtain the management object for networks
    network_client = NetworkManagementClient(credential, subscription_id)

    # Provision the virtual network and wait for completion
    poller = network_client.virtual_networks.begin_create_or_update(RESOURCE_GROUP_NAME,
        VNET_NAME,
        {
            "location": location,
            "address_space": {
                "address_prefixes": ["10.0.0.0/16"]
            }
        }
    )

    vnet_result = poller.result()

    print(f"Provisioned virtual network {vnet_result.name} with address prefixes {vnet_result.address_space.address_prefixes}")

    # Step 3: Provision the subnet and wait for completion
    poller = network_client.subnets.begin_create_or_update(RESOURCE_GROUP_NAME, 
        VNET_NAME, SUBNET_NAME,
        { "address_prefix": "10.0.0.0/24" }
    )
    subnet_result = poller.result()
    print(f"Provisioned virtual subnet {subnet_result.name} with address prefix {subnet_result.address_prefix}")

    # Step 4: Provision an IP address and wait for completion
    poller = network_client.public_ip_addresses.begin_create_or_update(RESOURCE_GROUP_NAME,
        IP_NAME,
        {
            "location": location,
            "sku": { "name": "Standard" },
            "public_ip_allocation_method": "Static",
            "public_ip_address_version" : "IPV4"
        }
    )

    ip_address_result = poller.result()

    print(f"Provisioned public IP address {ip_address_result.name} with address {ip_address_result.ip_address}")

    # Step 5: Provision the network interface client
    poller = network_client.network_interfaces.begin_create_or_update(RESOURCE_GROUP_NAME,
        NIC_NAME, 
        {
            "location": location,
            "ip_configurations": [ {
                "name": IP_CONFIG_NAME,
                "subnet": { "id": subnet_result.id },
                "public_ip_address": {"id": ip_address_result.id }
            }]
        }
    )

    nic_result = poller.result()
    print(f"Provisioned network interface client {nic_result.name}")

    # Step 6: Provision the virtual machine

    # Obtain the management object for virtual machines
    compute_client = ComputeManagementClient(credential, subscription_id)

    VM_NAME = "ExampleVM"
    USERNAME = "azureuser"
    PASSWORD = "ChangePa$$w0rd24"

    print(f"Provisioning virtual machine {VM_NAME}; this operation might take a few minutes.")
    # Provision the VM specifying only minimal arguments, which defaults to an Ubuntu 18.04 VM
    # on a Standard DS1 v2 plan with a public IP address and a default virtual network/subnet.

    poller = compute_client.virtual_machines.begin_create_or_update(RESOURCE_GROUP_NAME, VM_NAME,
        {
            "location": location,
            "storage_profile": {
                "image_reference": {
                    "publisher": 'Canonical',
                    "offer": "UbuntuServer",
                    "sku": "16.04.0-LTS",
                    "version": "latest"
                }
            },
            "hardware_profile": {
                "vm_size": "Standard_DS1_v2"
            },
            "os_profile": {
                "computer_name": VM_NAME,
                "admin_username": USERNAME,
                "admin_password": PASSWORD
            },
            "network_profile": {
                "network_interfaces": [{
                    "id": nic_result.id,
                }]
            }        
        }
    )
    vm_result = poller.result()
    print(f"成功创建虚拟机 {vm_result.name}")

    print("通过ssh的方式连接使用虚拟机")

def _ssh_init_ranchar(host,password,username="code",port=22,):
    ssh = paramiko.SSHClient() 
    # 允许连接不在~/.ssh/known_hosts文件中的主机
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())    
    # 连接服务器
    ssh.connect(hostname=host, port=port, username=username, password=password)    
    # 执行命令，不要执行top之类的在不停的刷新的命令(可以执行多条命令，以分号来分隔多条命令)
    # stdin, stdout, stderr = ssh.exec_command("cd %s;mkdir %s" % ("/www/wwwroot", "aa"))
    stdin, stdout, stderr = ssh.exec_command("ping 8.8.8.8 -c 10")
    stdin.write("终端等待输入...\n")   # test.py文件有input()函数，如果不需要与终端交互，则不写这两行
    stdin.flush()



import asyncio
import json
import logging
import os
from asyncore import write
from functools import wraps
from pathlib import Path

import yaml

from mtlibs import process_helper
# from mtlibs.container_config import registerService
# 注册服务
from mtlibs.openvpnClientProc import OpenvpnClientProc
from mtlibs.openvpnProc import OpenvpnProc
from mtlibs.tor_helper import TorProc

logger = logging.getLogger(__name__)
serviceHandlers = {}
# 正在运行的服务列表
servicelist = []

CONFIG_FILE_PATH = os.environ.get("CONFIG_PATH", "/app/config.yml")


def setConfigText(config_yaml_text):
    with open(CONFIG_FILE_PATH, 'w') as f:
        f.write(config_yaml_text)


def readConfig():
    """读取配置文件"""
    try:
        if not Path(CONFIG_FILE_PATH).exists():
            logger.error("错误，配置文件 '%s' 不存在" % CONFIG_FILE_PATH)
            return None
        with open(CONFIG_FILE_PATH, 'r') as f:
            return yaml.load(f.read())
    except Exception as e:
        print("出错", str(e))
        logger.exception(e)


def registerService(servicename):
    """修饰器，将函数声明为服务"""

    def aop(func):

        @wraps(func)
        def wrap(*args, **kwargs):
            print('before ' + str(servicename))
            func(*args, **kwargs)
            print('after ' + str(servicename))

        serviceHandlers.update({servicename: func})
        return wrap

    return aop


def getService(name):
    coroutin = serviceHandlers.get(name, None)
    if not coroutin:
        raise "任务内部的函数不存在"
    return coroutin


async def entry_script():
    """附加脚本"""
    logger.debug("[entry_script 开始]")
    config = readConfig()
    entry = config.get("entry", None)
    if entry:

        # with open(os.path.abspath("./entry"),"w") as f:
        #     f.write(entry)
        # Path(os.path.abspath("./entry")).chmod(0o7)

        await process_helper.subprocess_shell(entry)
    logger.debug("[ entry_script 结束]")


async def startAllService():
    """根据配置文件，对整个容器在启动阶段进行初始化"""
    logger.debug("[startAllService 开始]")
    config = readConfig()
    services = config.get("services", None)
    if services:
        for key in services.keys():
            logger.info("服务 \t{}\t==> {}".format(key, services[key]))
            handler = getService(key)
            serverArgs = services[key]
            servicelist.append({"cor": handler, "args": serverArgs})
        for ser in servicelist:
            handler = ser['cor']
            args = ser["args"]
            await handler(args)
    logger.debug("[ startAllService 结束]")


def getEnv(key):
    value = os.environ.get(key, None)
    if value:
        return value
    config = readConfig()
    env = config.get("env")
    return env.get(key, None)


@registerService("vpnclient")
async def vpnclient(serviceArgs):
    await OpenvpnClientProc(ovpn=serviceArgs['ovpn']).start()


@registerService("vpnserver")
async def vpnserver(serviceArgs):
    # TODO: 将1194tcp 也一同打开。提供两个udp/tcp两个协议链接方式，这样兼容性更高。
    # 要让客户端通过vpn服务器链接外网，必须设置nat
    # 注意脚本中的IP要跟实际情况对应
    script = """
echo 1 > /proc/sys/net/ipv4/ip_forward
#vpn内网网段（即客户端所在的网段允许通过nat连接外网）
iptables -t nat -A POSTROUTING -s 192.168.0.0/16 -o eth0 -j MASQUERADE
"""
    process_helper.bash(script)
    await OpenvpnProc().start()


@registerService("rdp")
async def rdp(serviceArgs):
    """启动远程桌面服务(vnc)
        注意，当前没细致设置相关功能和优化
    """
    process_helper.bash("""mkdir ~/.vnc
echo "vscode" | vncpasswd -f >> ~/.vnc/passwd
chmod 600 ~/.vnc/passwd
vncserver :1 -localhost no""")


@registerService("hiddenservice")
async def start(serviceArgs):
    private_key = serviceArgs.get("private_key", None)
    hiddenservices = serviceArgs.get("hiddenservices", [])
    socks_port = serviceArgs.get("socks_port", None)
    logger.info("hiddenservice private key {}".format(private_key))
    logger.info("hiddenservices {}".format(json.dumps(hiddenservices)))
    torinfo = await TorProc().start(socks_port=socks_port,
                                    b64_hs_ed25519_secret_key=private_key,
                                    hiddenservice=hiddenservices)
    logger.info("tor info {}".format(torinfo))



#!/usr/bin/env python3
import logging
import paramiko
import os
from pathlib import Path
from paramiko import SSHClient
import shutil
# from mtlibs.sshClient import SshClient
# from mtlibs.SSH import SSH
from dotenv import load_dotenv, find_dotenv
import logging
logging.basicConfig(level = logging.DEBUG,format = '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)


ENV_FILE = find_dotenv()
if ENV_FILE:
    load_dotenv(ENV_FILE)

load_dotenv(".env")


def ngrok_install_ifneed():
    ngrok = shutil.which("ngrok")
    if not ngrok:
        os.system("""curl -s https://ngrok-agent.s3.amazonaws.com/ngrok.asc | sudo tee /etc/apt/trusted.gpg.d/ngrok.asc >/dev/null && echo "deb https://ngrok-agent.s3.amazonaws.com buster main" | sudo tee /etc/apt/sources.list.d/ngrok.list && sudo apt update && sudo apt install ngrok""")
        
def ngrok_up_ssh():
    ngrok_install_ifneed()
    ngrok_authtoken = os.environ.get("NGROK_AUTHTOKEN")
    logger.info(f"ngrok token: {ngrok_authtoken}")
    # os.system(f"ngrok config add-authtoken {ngrok_token}")
    os.system(f"ngrok --authtoken {ngrok_authtoken} tcp 22")
    
def main():
    ngrok_up_ssh()
    
if __name__ == "__main__":
    ngrok_up_ssh()
    


#!/usr/bin/env python3



集成mitmproxy前的备份。











import os,sys
import subprocess
DEFAULT_PASSWORD='feihuo321'
trans_port=7892
# print("vpnhelper.py",sys.argv)
print("注意，当前使用iptables-legacy 而不是nf-tables")
if 'up' == os.environ.get('script_type'):
    # 服务器启动事件
    print("服务器启动。。。。。。。。。。。。。。。。。。。。。")
elif 'down' == os.environ.get('script_type'):
    # 服务器启动事件
    print("服务器关闭。。。。。。。。。。。。。。。。。。。。。")

elif 'user-pass-verify' == os.environ.get('script_type'):
    # 登陆认证
    username = os.environ.get('username')
    password = os.environ.get('password')
    print(os.environ)
    print(f"用户名：{username}")
    if not password == DEFAULT_PASSWORD:
        print(f"登陆失败,密码不正确， {username}")
        exit(1)
    else:
        print(f"密码不正确{username} ")
        exit(0)

elif 'client-connect' == os.environ.get('script_type'):
    # 新的客户端连接
    username = os.environ.get('username')
    ifconfig_pool_remote_ip = os.environ.get('ifconfig_pool_remote_ip')
    dev = os.environ.get('dev')
    route_network_1 = os.environ.get('route_network_1')
    route_network_2 = os.environ.get('route_network_2')
    print(f"用户名: {username}, dev {dev}, IP: {ifconfig_pool_remote_ip}")
    print(f"route_network_1: {route_network_1}, route_network_2 {route_network_2}")
    # 流量转发到clash透明代理端口

    iptable_cmd = f"""iptables -t nat -A PREROUTING -s {ifconfig_pool_remote_ip} -p tcp --syn -j REDIRECT --to-ports {trans_port}"""
    print(iptable_cmd)
    #########################################################################################################
    ## 坑：如果是根clash搭配，53端口没必要nat转发到5353.因为clash已经有相关的设置(clash.yaml dns字段)。
    subprocess.check_call(iptable_cmd, shell=True)
    # 删除 iptables-legacy -t nat -D PREROUTING -s 10.12.12.202 -p udp --dport 53 -j REDIRECT --to-ports 5353
    # iptables-legacy -t nat -D PREROUTING -s 10.12.12.202 -p tcp --syn -j REDIRECT --to-ports 7892

elif 'client-disconnect' == os.environ.get('script_type'):
    # 客户端断开  
    username = os.environ.get('username')
    ifconfig_pool_remote_ip = os.environ.get('ifconfig_pool_remote_ip')
    dev = os.environ.get('dev')
    route_network_1 = os.environ.get('route_network_1')
    route_network_2 = os.environ.get('route_network_2')
    print(f"用户名: {username}, dev {dev}, IP: {ifconfig_pool_remote_ip}")
    print(f"route_network_1: {route_network_1}, route_network_2 {route_network_2}")
    # 删除NAT iptables 规则
    print(f"删除客户端的iptables 规则 {ifconfig_pool_remote_ip}")
    subprocess.check_call(f"""
    #iptables -t nat -D PREROUTING -s {ifconfig_pool_remote_ip} -p udp --dport 53 -j REDIRECT --to-ports 5353
    iptables -t nat -D PREROUTING -s {ifconfig_pool_remote_ip} -p tcp --syn -j REDIRECT --to-ports {trans_port}
    """, shell=True)




import sys,os,time
import asyncio
import logging
import shlex
import subprocess
import threading

logger = logging.getLogger(__name__)


def process_output(process):
    while True:
        output = process.stdout.readline().decode()
        if process.poll() is not None:
            break
        if output:
            print(output, flush=True)


def process_output_threadhandler(process):
    """用线程的方式将进程的输出显示到终端上"""

    def _handler(process):
        while True:
            result = process.poll()
            if result is not None:
                print(f'进程结束:{result}')
                break
            else:
                output = process.stdout.readline().decode()
                if output:
                    print(output.rstrip(), flush=True)
                if process.stderr:
                    err = process.stderr.readline().decode()
                    if err:
                        print(f"stderr, {err}", flush=True)

    threading.Thread(target=_handler, args=(process, )).start()


def exec_cmd(cmd):
    """
        助手函数, 启动系统进程,并直接从sys.stdout 回显
    """

    def handle_output(process):
        while True:
            output = process.stdout.readline().decode()
            if process.poll() is not None:
                break
            if output:
                logger.info(output)
                # sys.stdout.flush()

    process = subprocess.Popen(shlex.split(cmd), stdout=subprocess.PIPE)
    threading.Thread(target=handle_output, args=(process, )).start()


def exec_cmd2(cmd, output_file="/var/log/clilog.log"):
    """
        助手函数, 启动系统进程,并将进程的输出写入到文件中
        TODO: 应该升级为流模式, 这样就不用关日志是不是文件,可以使用StringIO
    """

    def handle_output(process):
        try:
            with open(output_file, 'ab+') as logfile:
                while True:
                    if process.stdout:
                        bytes = process.stdout.read()
                        logfile.write(bytes)
                        logfile.flush()
                    if process.stderr:
                        bytes = process.stderr.read()
                        logfile.write(bytes)
                        logfile.flush()

                    if process.poll() is not None:
                        break
        except Exception as e:
            print(e)

    process = subprocess.Popen(shlex.split(cmd), stdout=subprocess.PIPE)
    threading.Thread(target=handle_output, args=(process, )).start()


def exec(cmd, check=True):
    """
    执行命令,并返回命令的输出作为返回值
    如果命令执行失败,会抛出异常
    """
    logger.info("exec>{}".format(cmd))
    # print("exec env:")
    # print(os.environ)
    return subprocess.run(shlex.split(cmd), check=check, env=os.environ)


def bash(shelltext):
    """
    运行shell脚本并，等到进程结束后，返回执行结果
    说明： 推荐使用异步的方式，本函数是同步的方式，感觉有点不方便。
    """
    proc = subprocess.Popen(shlex.split("bash -"),
                            shell=True,
                            stdin=subprocess.PIPE,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.STDOUT)
    # communicate 会等待执行完毕,获取stdout,和 stderr
    return proc.communicate(input=shelltext.encode())


async def subprocess_shell(script):
    """以异步的方式运行shell"""
    # logger.info("[subprocess_shell 开始]")
    proc = await asyncio.create_subprocess_shell(
        script,
        stdout=asyncio.subprocess.PIPE,
        stderr=asyncio.subprocess.STDOUT)
    # 循环读取，能达到效果，更高级的写法是使用protocol
    # 参考：http://songcser.github.io/2017/10/26/working-with-subprocesses/
    line = await proc.stdout.readline()
    while line:
        logger.info(line.decode().strip())
        line = await proc.stdout.readline()


def is_tool(name):
    """Check whether `name` is on PATH."""
    from distutils.spawn import find_executable
    return find_executable(name) is not None



import os
import sys
from time import sleep

from beeprint import pp

# ########################################################################################################
# 此版本还在设计中。
# 目的是提供一个统一的文件系统，文件系统应该能兼容云端S3之类的。
# 那么，对于本地的文件处理，应该有唯一的根目录。这样好方便同一处理，备份，同步等问题。
# 初步的编程想法：
#         看是否有合适的第三方库。
#         看是否合适使用 provider 模式

# 文件系统跟目录
LOCAL_ROOT = "/mtcli/data"


def writeText(filepath):
    """写入文件"""
    open(filepath, 'w')


def get_filepaths_by_walk(basedir, curr_dir=None, result=set()):
    if not curr_dir:
        curr_dir = basedir
    # print(f"curr_dir {curr_dir}")
    curr_dir = (curr_dir + "/").replace("//", "/")

    if not os.path.isdir(curr_dir):
        return
    dirlist = os.walk(curr_dir)
    for parent_dir, dirs, files in dirlist:
        for file in files:
            # print(os.path.join(root, file))
            result.add(os.path.join(parent_dir, file)[len(basedir) + 1:])
        for _dir in dirs:
            # print(f"_dir: {_dir}")
            next = os.path.join(curr_dir, _dir)
            # print(f"遍历下级：{next}")
            # sleep(1)
            get_filepaths_by_walk(basedir, next, result)

    return result



import asyncio
import os
from git.repo import Repo
from pathlib import Path
from csservices import portforward
import time
 

WORKSPACE_ROOT = os.environ.get("WORKSPACE_ROOT", "/workspace2")
print(f"从环境变量中获取到：WORKSPACE_ROOT: {WORKSPACE_ROOT}")
AZ_REPO_PASSWORD = os.environ.get("AZ_REPO_PASSWORD")# "x2hx4x3hpmrmf2xx7hrtieaxlovfwqorqzb3nyoh75eyqevs6dca"
AZ_REPO_USER = os.environ.get("AZ_REPO_USER")

async def on_command(args):
    """启动容器作为开发环境"""
    print("dev 参数：", args)
    project = args.project
    repo = args.repo
    print(f"准备源码 {project}, {repo}",flush=True)
    azure_gitclone(project,repo)
    print("就绪",flush=True)
    while True:
        time.sleep(1)

def azure_gitclone(project, repo, branch="master"):
    """clone 位于azure 上的仓库源码"""
    local_path = f"{WORKSPACE_ROOT}/{project}/{repo}"
    if Path(local_path).exists():
        print(f"目标路径{local_path}存在，跳过gitclone")
    else:
        print(f"本地路径${local_path}")
        Repo.clone_from(
            f"https://{AZ_REPO_USER}:{AZ_REPO_PASSWORD}@dev.azure.com/{AZ_REPO_USER}/{project}/_git/{repo}",
            to_path=local_path,
            # branch=branch
            )
        print(f"克隆{project}.{project}完成", flush=True)


import asyncio
import logging
import os
import shlex
import subprocess
import tempfile
import traceback
from mtlibs import mtutils, process_helper
from mtlibs.mtutils import ranstr


logger = logging.getLogger(__name__)


class OpenvpnProc():
    """
        openvpn 服务进程
        说明： openvpn 服务入口点是openvpn.conf配置文件。
            而配置文件里面需要指定好几个证书，而且，还有客户端证书部分的处理比较复杂繁琐。
            目前的解决办法是，通过其他方式预先将证书生成。放到指定路径下。
            用：kylemanna/docker-openvpn
            1:添加客户端证书
            docker run -v /code/cli/data/openvpn:/etc/openvpn --rm -it kylemanna/openvpn easyrsa build-client-full client2 nopass
            2：获取客户端配置文件
            docker run -v /code/cli/data/openvpn:/etc/openvpn --rm -it kylemanna/openvpn ovpn_getclient main_win > /code/cli/data/ovpn/main_win.ovpn


        openvpn配置文件字段说明：
            [verb]: 日志输出的详细程度，9为最高，
    """

    def __init__(self):
        self.tmpdirname = tempfile.gettempdir() + "/openvpn_" + ranstr(20)

    def write_openvpnconfig(self):
        self.pki_base_path = os.path.abspath("./resources/gateway/openvpn/pki")
        self.ccd_path = os.path.abspath("./resources/gateway/openvpn/ccd")
        self.openvpn_config_path = self.tmpdirname + "/openvpn.conf"
        mtutils.writefile(self.openvpn_config_path, """
server 192.168.255.0 255.255.255.0
verb 3
key {key}
ca {ca_path}
cert {cert}
dh {dh_path}
tls-auth {tls_auth}
key-direction 0
keepalive 10 60
persist-key
persist-tun

proto udp
# Rely on Docker to do port mapping, internally always 1194
port 1194
dev tun0
status /tmp/openvpn-status.log

user nobody
group nogroup
comp-lzo no

client-to-client
verb 3
client-config-dir {ccd_path}

### Route Configurations Below
route 192.168.255.0 255.255.255.0
# route 75.126.135.131 255.255.255.255
#成功连接后添加主机路由信息(能让客户端自动设置路由表)
push "route 192.168.254.0 255.255.255.0"
#下面注释了的，是特定ip的路由设置（客户端）
# push "route 8.8.8.8 255.255.255.255"
# push "route 75.126.135.131 255.255.255.255"
### Push Configurations Below
# push "block-outside-dns"
push "dhcp-option DNS 8.8.8.8"
push "dhcp-option DNS 8.8.4.4"
push "comp-lzo no"

""".format(
            key=os.path.join(self.pki_base_path,
                             "private/VPN.SERVERNAME.COM.key"),
            ca_path=os.path.join(self.pki_base_path, "ca.crt"),
            dh_path=os.path.join(self.pki_base_path, "dh.pem"),
            tls_auth=os.path.join(self.pki_base_path, "ta.key"),
            cert=os.path.join(self.pki_base_path,
                              "issued/VPN.SERVERNAME.COM.crt"),
            ccd_path=self.ccd_path,
        ).encode())

    async def handle_output(self):
        try:
            while True:
                if self.process.stdout:
                    line = self.process.stdout.readline().decode().strip()
                    logger.debug("vpn->{}".format(line))
                    # if line.strip().find('Initialization Sequence Completed') > 0:
                    #     return "ok"
                    # TODO: 捕获失败的字符串，返回
                    # 要不然，进程启动失败会卡住,协程一直不退出
                if self.process.stderr:
                    line = self.process.stderr.readline().decode()
                    logger.info("vpn->[error]{}".format(line))
                if self.process.poll() is not None:
                    break
        except Exception as e:
            logger.error(traceback.format_exc(e))

    async def start(self):
        """启动openvpn进程"""
        self.write_openvpnconfig()
        # 必要的初始化

        bash_init = """
mkdir -p /dev/net
if [ ! -c /dev/net/tun ]; then
    mknod /dev/net/tun c 10 200
fi"""
        process_helper.bash(bash_init)
        cmd = "openvpn --config " + self.openvpn_config_path
        self.process = subprocess.Popen(
            shlex.split(cmd), stdout=subprocess.PIPE)
        await self.handle_output()



import zipfile
import os
import base64
from  pathlib import Path
from mtlibs.github_helper import Ghrepo
repo = Ghrepo(url='https://xbintor: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@github.com/xbintor/p')
IMGTAG='cli'
def on_command(args):
    """构建"""   
    # GHOWNER='xbintor'
    # GHTOKEN='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'
    

    # ghurl = "https://xbintor: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@github.com/xbintor/p"

    def ToBase64(file):
        with open(file, 'rb') as fileObj:
            image_data = fileObj.read()
            base64_data = base64.b64encode(image_data)
            fout = open(txt, 'w')
            fout.write(base64_data.decode())
            fout.close()
            


    print('构建压缩文件')
    zfName = './dist/mydata.zip'
    Path(zfName).parent.mkdir(exist_ok=True)
    
    foo = zipfile.ZipFile(zfName, 'w')
    for root, dirs, files in os.walk('data'):
        for f in files:
            foo.write(os.path.join(root, f))
    foo.close()

    print("生成base64py文件")

    with open(zfName, 'rb') as f:
        bin = f.read()
        base64Str = base64.b64encode(bin)
        pySrcCode = f'''def getZipBase64():
        return '{base64Str.decode()}'
    '''
    
    # with open('datazipSrc.py','w') as f:
    #     f.write(pySrcCode)

    # os.system('pyinstaller -F main.py -n main -i favicon.ico')
    # with open('./dist/main','rb') as f:
    #     repo.write_content('main',f.read())
    # os.system(f'echo {repo.token} | docker login ghcr.io -u USERNAME --password-stdin')    
    # os.system(f'docker build -t ghcr.io/{repo.owner}/{IMGTAG} .')
    # os.system(f'docker push ghcr.io/{repo.owner}/{IMGTAG}')


def pubgitrepo():
    """将编译好的发布到github仓库文件中（单个文件）"""
    with open('./dist/main','rb') as f:
        repo.write_content('main',f.read())

    # pyinstaller -F main.py -n main -i favicon.ico
    


#!/usr/bin/env python3
import sys
import os
from mtlibs.docker_helper import isInContainer
from mtlibs import process_helper
from pathlib import Path
import logging
from dotenv import load_dotenv, find_dotenv
import json
import logging
import shlex
import subprocess
import time
from os.path import relpath
from typing import Dict, List, Tuple
from urllib.parse import urlparse
import re
from os import path
import argparse
from mtlibs.github import gitclone,gitParseOwnerRepo
ENV_FILE = find_dotenv()
if ENV_FILE:
    load_dotenv(ENV_FILE)

load_dotenv(".env")

import logging
logging.basicConfig(level = logging.DEBUG,format = '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)


def check_env():
    env_names = [
        "GIT_TOKEN",
        "DOCKER_HUB_USER",
        "DOCKER_HUB_PASSWORD",
    ]

    # print(os.environ)
    for item in env_names:
        print(f"---{item}")        
        if not os.environ.get(item):            
            print(f"error: need env {item}")
            return False

    return True
    

def gitpod_clone_private_repo(giturl:str):
    """
        给定git 网址，下载到指定路径并根据规则运行相关代码。
    """
    logger.info(f"giturl {giturl}")
    parsed = urlparse(giturl)
    if not parsed.hostname:
        logger.error(f"url incorrect : {giturl}")
        return
    uri = parsed
    owner,repo,file = gitParseOwnerRepo(giturl)

    clone_to = path.join("/workspace",repo)
    gitclone(owner,repo,parsed.username,clone_to)
    if not file:
        logger.info("no entry script,skip launch")
    if file:
        file = file.lstrip("/")
        scriptFile = path.join(clone_to,file)
        if not Path(scriptFile).exists():
            logger.warn(f"入口文件不存在{scriptFile}")

        Path(scriptFile).chmod(0o700)
        logger.info(f"[TODO]开始执行入口文件 {scriptFile}") 
        # logfile = open(path.join(clone_to,GITUP_LOGFILE),'w')
        # subprocess.Popen([scriptFile],
        #     stdout=logfile, #输出重定向到文件。
        #     stderr=logfile,
        #     # stdout=subprocess.PIPE, 
        #     # stderr=subprocess.PIPE,
        #     cwd=clone_to,
        #     shell=False,
        # )
def main():
    print(f"mtgp 这个好像没用了。。。。。。")
    # parser = argparse.ArgumentParser()
    # parser.add_argument("urls") 
    # args = parser.parse_args()
    # logger.info(f"urls: {args.urls}")
    # logger.info(f"urls: {args.urls}")
    # gitup_urls = args.urls or os.environ.get("MTX_GITUP")
    # if not gitup_urls:
    #     logger.info(f"need urls")
    #     exit()
    # items = gitup_urls.split("|")
    # for item in items:
    #     gitpod_clone_private_repo(item)

if __name__ == '__main__':
    main()


#!/usr/bin/env python3
import logging
import paramiko
import os
from pathlib import Path
from paramiko import SSHClient
from mtlibs.sshClient import SshClient
from mtlibs.SSH import SSH
from dotenv import load_dotenv, find_dotenv
import argparse
import logging
from mtlibs.yaml import load_yaml_file
import json

APP_NAME = "mtdp"

ENV_FILE = find_dotenv()
if ENV_FILE:
    load_dotenv(ENV_FILE)

load_dotenv(".env")

import logging
logging.basicConfig(level = logging.DEBUG,format = '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)

def loadConfig():
    config = load_yaml_file(os.path.join(f"./deploy/{APP_NAME}.yml"))
    # logger.info(f"config data: {config}")
    mtdp_config  = config[APP_NAME]
    if not mtdp_config:
        raise Exception("配置文件不正确")
    return mtdp_config

def sub_default(args):
    mtdp_config = loadConfig()
    logger.info(f"配置：{mtdp_config}")
    
    
    ssh_config = mtdp_config["target"]["config"]
    user_name = ssh_config["username"]
    password = ssh_config["password"]
    host=ssh_config["host"]
    port = ssh_config["port"]
    
    logger.info("connect begin")
    client = SshClient(host,port=port, password=password, user=user_name)
    connect_success = client.connect()
    if not connect_success:
        logger.debug(f"连接失败 {user_name}@{host}:{port}")
        
    # 范例：执行命令
    cmd = "ls -al /"
    output = client.exec_cmd(cmd)
    logger.info(f"输出 {output}")
    
    # # demo: 下载整个文件夹
    # remotefile, local_file = '/.dockerenv', '.tmp/dockerenv1'
    # client.sftp_get(remotefile, local_file)  # 下载文件    
    # client.sftp_get_dir("/root",".tmp/test1")   
	
    # 上传文件测试
    remotedir = "/app2"
    client.exec_cmd(f"mkdir {remotedir}")
    localdir = os.path.join(os.getcwd(),"deploy/uploads")    
    client.sftp_put_dir(localdir,remotedir)  # 上传文件夹
    
    # 释放资源
    client.close()
    
def main():
    print("mtdp  这个命令好像没用了。")
    parser = argparse.ArgumentParser(description="mtdp")
    parser.add_argument('dir',default=".", nargs="*")       
    #设置默认函数
    parser.set_defaults(func=sub_default)
    args = parser.parse_args()
    #执行函数功能
    args.func(args)


if __name__ == "__main__":
    main()
    



import os
import sys
import time
import subprocess
from subprocess import run, Popen, PIPE
from pathlib import Path
from urllib.parse import urlparse
import shutil
import logging
from mtlibs.process_helper import is_tool
logger = logging.getLogger(__name__)


class Nginx:

    def __init__(self, base_dir):
        self.base_dir = base_dir

    def setup_nginx_django_init_site(self, 
        domain_name="localhost", 
        with_php=True,
        # with_wordpress=False,
        ):
        """
            由于环境不同，可能不能立即获取到证书，所以先启动一个http的 确保django能启动
        """

        logger.info(f"启动主站，绑定域名:{domain_name}")
        cert_dir = f"/etc/letsencrypt/live/{domain_name}"
        if Path("/etc/nginx/conf.d/default.conf").exists():
            shutil.move("/etc/nginx/conf.d/default.conf", "/etc/nginx/conf.d/default.backup")


        nginx_conf_tpl = """
server {
    root @@www_root@@; 
    client_max_body_size 256M;
    index index.html index.htm index.php;
    server_name @@DOMAIN_NAME@@;   

    # 其他页面先尝试相关文件和路径，由前端nextjs兜底
    location / {
        # try_files $uri $uri/ /index.php?$args @backend;
        try_files $uri $uri/ /index.php?$args;
    }            

    location = /favicon.ico { access_log off; log_not_found off; }
    location = /robots.txt  { access_log off; log_not_found off; }
    access_log /var/www/logs/access.log;
    error_log  /var/www/logs/error.log error;
    # 无任何页面时，默认路由，也许这样的方式作为兜底页面，是一个不错的选择
    #error_page 404 /index.php;
    location ^~ /mtxadmin/ {
        add_header X-Powered-By 'PHP';
        # try_files @nextfront $uri $uri/;
        proxy_pass http://backend;
        # autoindex on;
        # index index.html index.htm;
    }
    location ^~ /static/ {
        # 开发板的静态文件
        add_header X-Powered-By 'PHP';
        proxy_pass http://backend;
    }
    location ^~ /api/ {
        add_header X-Powered-By 'PHP';
        proxy_pass http://backend;
    }

    location @backend {
        proxy_pass http://backend;
        proxy_http_version 1.1;
        proxy_set_header Upgrade $http_upgrade;
        proxy_set_header Connection 'upgrade';
        proxy_set_header Host $host;
        proxy_cache_bypass $http_upgrade;
    }

    # Necessary for Let's Encrypt Domain Name ownership validation. Place any other deny rules after this
    location ~ /.well-known {
        allow all;
    }

    # Deny access to .htaccess or .htpasswd files
    location ~ /\.ht {
        deny all;
    }

    # Deny access to any git repository
    location ~ /\.git {
        deny all;
    }
    @@PHP@@

    listen 80;
    @@LISTEN@@
    @@REDIRECT_HTTPS@@

    
}

"""

        # DOMAIN_NAME = domain_name or os.environ.get("DOMAIN_NAME", "localhost")
        # print(f"DOMAIN_NAME: {DOMAIN_NAME}")
        # cert_dir = self.renew_cert(DOMAIN_NAME)
        # if not cert_dir:
        #     logger.info(f"证书没有成功获取")
        #     return



        nginx_php = ""
        if with_php:
            nginx_php="""
    location ~ \.php$ {
        fastcgi_split_path_info ^(.+\.php)(/.+)$;
        # fastcgi_pass unix:/var/run/php/php8.1-fpm.sock;
        fastcgi_pass 127.0.0.1:9000;
        fastcgi_index index.php;
        include fastcgi_params;
        fastcgi_param SCRIPT_FILENAME $document_root$fastcgi_script_name;
        fastcgi_intercept_errors off;
        fastcgi_buffer_size 16k;
        fastcgi_buffers 4 16k;
        fastcgi_connect_timeout 600;
        fastcgi_send_timeout 600;
        fastcgi_read_timeout 600;
    }
"""     
        nginx_listen = ""

        logger.info(f"尝试找域名证书: {cert_dir}")
        
            
        if domain_name != "localhost" and cert_dir and Path(os.path.join(cert_dir,"fullchain.pem")).exists():
            # 如果设置了绑定域名，使用SSL证书
            nginx_listen=f"""
    listen 443 ssl;
    ssl_certificate {cert_dir}/fullchain.pem;
    ssl_certificate_key {cert_dir}/privkey.pem;
    include /etc/letsencrypt/options-ssl-nginx.conf;
    ssl_dhparam /etc/letsencrypt/ssl-dhparams.pem;
"""     
        else:
            logger.info(f"域名: [{domain_name}]")
            if domain_name != "localhost":
                logger.info(domain_name)
                logger.info("========================")
                try:
                    logger.info(f"尝试获取, letsencrypt 证书，域名:{domain_name}")
                    self.renew_cert(domain_name)
                except Exception as e:
                    logger.info(f"获取letsencrypt证书失败, 域名：{domain_name},详情：")
                    logger.exception(e)
                # 使用临时证书，先确保站点能以https的方式启动。
                temp_cert_dir =os.path.join( os.getcwd(), "certs/localhost")
                nginx_listen=f"""
        listen 443 ssl;
        ssl_certificate {temp_cert_dir}/server.crt;
        ssl_certificate_key {temp_cert_dir}/server.key;
        # include /etc/letsencrypt/options-ssl-nginx.conf;
        # ssl_dhparam /etc/letsencrypt/ssl-dhparams.pem;
"""     
                logger.info("使用临时证书，先确保站点能以https的方式启动。")
        

        nginx_redirect_https = """if ($scheme = http) {
        return 301 https://$host$request_uri;
    }"""
        logger.info(f"==nginx_redirect_https {nginx_redirect_https}")
        if os.environ.get("GITPOD_WORKSPACE_ID"):
            logger.info(f"在gitpod环境下,无需https跳转")
            nginx_redirect_https = ""

        site_root = os.path.join(self.base_dir, domain_name)
        nginx_conf = nginx_conf_tpl\
            .replace("@@DOMAIN_NAME@@", domain_name)\
            .replace("@@www_root@@", site_root)\
            .replace("@@PHP@@", nginx_php)\
            .replace("@@LISTEN@@", nginx_listen)\
            .replace("@@REDIRECT_HTTPS@@", nginx_redirect_https)

        with open(f"/etc/nginx/sites-enabled/{domain_name}", 'w') as fd:
            fd.write(nginx_conf)

        print(f"nginx.confg配置=============================================")
        print(nginx_conf)
        print("========================================================")
        # 设置目前权限
        if not os.path.exists(site_root):
            Path(site_root).mkdir(mode=0o700, exist_ok=True, parents=True)
        run(f"sudo chown nginx -R {site_root}", shell=True, check=True)
        run(f"nginx -t", shell=True, check=True)


    def setup_nginx(self):
        """根据环境变量和相关参数生产nginx配置文件"""
        nginx_conf_tpl = """user  www-data;
    worker_processes auto;
    error_log  /var/log/nginx/error.log notice;
    pid        /var/run/nginx.pid;
    events {
        worker_connections  1024;
    }
    http {
        include       /etc/nginx/mime.types;
        default_type  application/octet-stream;
        log_format  main  '$remote_addr - $remote_user [$time_local] "$request" '
                        '$status $body_bytes_sent "$http_referer" '
                        '"$http_user_agent" "$http_x_forwarded_for"';
        access_log  /var/log/nginx/access.log  main;
        sendfile        on;
        #tcp_nopush     on;
        keepalive_timeout  65;

        gzip  on;
        gzip_comp_level 5;

        # gzip.conf
        gzip_min_length 256;
        gzip_proxied any;
        gzip_vary on;    

        include /etc/nginx/conf.d/*.conf;
        include /etc/nginx/sites-enabled/*;

        # upstream front {
        #     # 默认前端
        #     server front:3000 weight=100 max_fails=12 fail_timeout=60s;
        # }

        upstream backend {
            server 127.0.0.1:8000 weight=450 max_fails=12 fail_timeout=60s;
        }

    }
    """

        nginx_conf = nginx_conf_tpl
        with open("/etc/nginx/nginx.conf", 'w') as fd:
            fd.write(nginx_conf)

        logger.info(f"初始化相关目录")
        os.makedirs("/var/www/logs", exist_ok=True)
        # os.path.exists(
        #     "/etc/nginx/conf.d") and shutil.rmtree("/etc/nginx/conf.d")
        Path("/etc/nginx/conf.d/").mkdir(exist_ok=True)
        Path("/etc/nginx/sites-enabled/").mkdir(exist_ok=True)


    def renew_cert(self, domain, exist_ok=True) -> bool:
        """重新获取ssl证书"""
        cert_dir = f"/etc/letsencrypt/live/{domain}"
        if exist_ok and os.path.exists(cert_dir):
            logger.info(f"ssl证书`{domain}`已经存在，跳过获取")
            return cert_dir

        else:
            if not is_tool('certbot'):
                run(f"sudo apt install -y certbot python3-certbot-nginx",
                    shell=True, check=True)
            # 使用nginx插件的形式获取证书，这个问题在于，需要nginx首先配置成为http的方式（即没有使用https的情况下）才行，考虑到容器经常重置的问题，这个方式不妥。
            # run(f"sudo certbot certonly --non-interactive --agree-tos --nginx -d {domain} -m a@a.com", shell=True, check=True)
            # 现在使用 certbot 独立的方式获取证书。获取证书后，直接手动配置nginx配置文件启动。
            certbot_email = os.environ.get("CERTBOT_EMAIL", "b@b.com")

            while True:
                try:
                    run(f"certbot certonly --non-interactive --agree-tos --standalone -d {domain} -m {certbot_email}",
                        shell=True,
                        check=True)
                    break
                except Exception as e:
                    logger.info(f"获取ssl证书出错, 60s 后重试, 错误消息:{str(e)}")
                    # logger.exception(msg)
                    time.sleep(60)

            # 确实证书文件存在
            if os.path.exists(cert_dir):
                return cert_dir

        logger.info(f"证书文件视乎不存在，请注意查找原因")
        return None

    def restart(self):
        """重启"""
        logger.info(f"重启nginx服务")
        run("sudo service nginx restart", shell=True)

    def new_site(self, domain_name):
        """添加新的站点配置"""
        nginx_conf_tpl = """server {
    if ($host = @@DOMAIN_NAME@@) {
        return 301 https://$host$request_uri;
    } # managed by Certbot


    server_name @@DOMAIN_NAME@@;
    listen 80;
    return 404; # managed by Certbot
}

server {
    root @@www_root@@; 
    client_max_body_size 256M;
    index index.html index.htm index.php;
    server_name @@DOMAIN_NAME@@;
    # location / {
    #     # First attempt to serve request as file, then
    #     # as directory, then fall back to displaying a 404.
    #     try_files $uri $uri/ /index.php?$query_string;
    # }

    # location / {
    #     # autoindex on;
    #     index index.html index.htm index.php;
    #     try_files $uri $uri/ /index.php?$args @front @backend;
    # }

    # 明确指定默认首页
    # location = / {
    #     proxy_pass http://front;
    #     proxy_http_version 1.1;
    #     proxy_set_header Upgrade $http_upgrade;
    #     proxy_set_header Connection 'upgrade';
    #     proxy_set_header Host $host;
    #     proxy_cache_bypass $http_upgrade;
    # }

    # 其他页面先尝试相关文件和路径，由前端nextjs兜底
    location / {
        try_files $uri $uri/ /index.php?$args @backend;
    }            

    location = /favicon.ico { access_log off; log_not_found off; }
    location = /robots.txt  { access_log off; log_not_found off; }
    access_log /var/www/logs/access.log;
    error_log  /var/www/logs/error.log error;
    # 无任何页面时，默认路由，也许这样的方式作为兜底页面，是一个不错的选择
    # error_page 404 /index.php;

    location ~ \.php$ {
        fastcgi_split_path_info ^(.+\.php)(/.+)$;
        # fastcgi_pass unix:/var/run/php/php8.1-fpm.sock;
        fastcgi_pass 127.0.0.1:9000;
        fastcgi_index index.php;
        include fastcgi_params;
        fastcgi_param SCRIPT_FILENAME $document_root$fastcgi_script_name;
        fastcgi_intercept_errors off;
        fastcgi_buffer_size 16k;
        fastcgi_buffers 4 16k;
        fastcgi_connect_timeout 600;
        fastcgi_send_timeout 600;
        fastcgi_read_timeout 600;
    }

    location ^~ /mtxadmin/ {
        add_header X-Powered-By 'PHP';
        # try_files @nextfront $uri $uri/;
        proxy_pass http://backend;
        # autoindex on;
        # index index.html index.htm;
    }
    location ^~ /static/ {
        # 开发板的静态文件
        add_header X-Powered-By 'PHP';
        proxy_pass http://backend;
    }
    location ^~ /api/ {
        add_header X-Powered-By 'PHP';
        proxy_pass http://backend;
    }

    location @backend {
        proxy_pass http://backend;
        proxy_http_version 1.1;
        proxy_set_header Upgrade $http_upgrade;
        proxy_set_header Connection 'upgrade';
        proxy_set_header Host $host;
        proxy_cache_bypass $http_upgrade;
    }

    # location @front {
    #     # reverse proxy for next server
    #     proxy_pass http://front;
    #     proxy_http_version 1.1;
    #     proxy_set_header Upgrade $http_upgrade;
    #     proxy_set_header Connection 'upgrade';
    #     proxy_set_header Host $host;
    #     proxy_cache_bypass $http_upgrade;
    # }

    # Necessary for Let's Encrypt Domain Name ownership validation. Place any other deny rules after this
    location ~ /.well-known {
        allow all;
    }

    # Deny access to .htaccess or .htpasswd files
    location ~ /\.ht {
        deny all;
    }

    # Deny access to any git repository
    location ~ /\.git {
        deny all;
    }

    # Deny access to xmlrpc.php - a common brute force target against Wordpress
    location = /xmlrpc.php {
        deny all;
        access_log off;
        log_not_found off;
        return 444;
    }

    listen 443 ssl; # managed by Certbot
    ssl_certificate /etc/letsencrypt/live/@@DOMAIN_NAME@@/fullchain.pem; # managed by Certbot
    ssl_certificate_key /etc/letsencrypt/live/@@DOMAIN_NAME@@/privkey.pem; # managed by Certbot
    include /etc/letsencrypt/options-ssl-nginx.conf; # managed by Certbot
    ssl_dhparam /etc/letsencrypt/ssl-dhparams.pem; # managed by Certbot
}"""

        DOMAIN_NAME = domain_name or os.environ.get("DOMAIN_NAME", "localhost")

        cert_dir = self.renew_cert(DOMAIN_NAME)
        if not cert_dir:
            logger.info(f"证书没有成功获取")
            return

        site_root = os.path.join(self.base_dir, DOMAIN_NAME)
        nginx_conf = nginx_conf_tpl\
            .replace("@@DOMAIN_NAME@@", DOMAIN_NAME)\
            .replace("@@www_root@@", site_root)

        
        with open(f"/etc/nginx/sites-enabled/{DOMAIN_NAME}", 'w') as fd:
            fd.write(nginx_conf)

        # print(f"nginx.confg配置=============================================")
        # print(nginx_conf)
        # print("========================================================")
        # 设置目前权限
        if not os.path.exists(site_root):
            Path(site_root).mkdir(mode=0o700, exist_ok=True, parents=True)
        run(f"sudo chown nginx -R {site_root}", shell=True, check=True)


    def forward(self, domain_name, target_url):
        """
            建立纯转发(反向代理)站点。
        """
        # target_uri = urlparse(target_url)
        nginx_conf_tpl = """
#转https
server {
    if ($host = @@DOMAIN_NAME@@) {
        return 301 https://$host$request_uri;
    } # managed by Certbot
    server_name @@DOMAIN_NAME@@;
    listen 80;
    return 404; # managed by Certbot
}
server {
    root @@www_root@@; 
    client_max_body_size 256M;
    server_name @@DOMAIN_NAME@@;
    location / {
        proxy_pass @@PROXY_PASS@@;
        # proxy_http_version 1.1;
        proxy_set_header Upgrade $http_upgrade;
        proxy_set_header Connection 'upgrade';
        proxy_set_header Host $host;
        proxy_cache_bypass $http_upgrade;
        proxy_set_header   X-Real-IP        $remote_addr;
        proxy_set_header   X-Forwarded-For  $proxy_add_x_forwarded_for;
        # 让scheme的值传递给后方，要不然，虽然页面时https，但是后端依然认为是http，导致可能的网址错误。
        proxy_set_header X-Forwarded-Proto $scheme;
    }
    listen 443 ssl; # managed by Certbot
    ssl_certificate /etc/letsencrypt/live/@@DOMAIN_NAME@@/fullchain.pem; # managed by Certbot
    ssl_certificate_key /etc/letsencrypt/live/@@DOMAIN_NAME@@/privkey.pem; # managed by Certbot
    include /etc/letsencrypt/options-ssl-nginx.conf; # managed by Certbot
    ssl_dhparam /etc/letsencrypt/ssl-dhparams.pem; # managed by Certbot
}"""
        cert_dir = self.renew_cert(domain_name)
        if not cert_dir:
            logger.info(f"证书没有成功获取")
            return

        nginx_conf = nginx_conf_tpl\
            .replace("@@DOMAIN_NAME@@", domain_name)\
            .replace("@@PROXY_PASS@@", target_url)
            # .replace("@@www_root@@", site_root)\

        with open(f"/etc/nginx/sites-enabled/{domain_name}", 'w') as fd:
            fd.write(nginx_conf)






# auth0 认证实用工具
#

import email
import json
import os

from django.contrib.auth import authenticate
import jwt
import requests
from django.conf import settings
from jwt import PyJWKClient
from django.contrib.auth.models import User
from rest_framework import authentication
from rest_framework import exceptions


def get_token_auth_header(request):
    """从http header 总获取access_token"""
    auth = request.META.get("HTTP_AUTHORIZATION", None)
    if auth:
        parts = auth.split()
        token = parts[1]
        return token
    return None

def jwt_get_username_from_payload_handler(payload):
    """
        提示：payload中，没有直接包含详细的用户信息，但是sub字段中包含了用户ID。
              代码是来自于官方范例。
    """
    username = payload.get('sub').replace('|', '.')
    authenticate(remote_user=username)
    return username


def jwt_decode_token(token):
    header = jwt.get_unverified_header(token)
    auth0_domain = settings.AUTH0_DOMAIN  # os.environ.get('AUTH0_DOMAIN')
    jwks = requests.get(
        'https://{}/.well-known/jwks.json'.format(auth0_domain)).json()
    public_key = None
    for jwk in jwks['keys']:
        if jwk['kid'] == header['kid']:
            public_key = jwt.algorithms.RSAAlgorithm.from_jwk(json.dumps(jwk))

    if public_key is None:
        raise Exception('Public key not found.')
    api_identifier = settings.API_IDENTIFIER

    issuer = 'https://{}/'.format(auth0_domain)
    return jwt.decode(token, public_key, audience=api_identifier, issuer=issuer, algorithms=['RS256'])


def jwt_decode_token2(token):
    """这个函数同jwt_decode_token， 只是不同的写法，留作参考。"""
    url = f'https://{settings.AUTH0_DOMAIN}/.well-known/jwks.json'
    jwks_client = PyJWKClient(url)
    signing_key = jwks_client.get_signing_key_from_jwt(token)
    return jwt.decode(
        token,
        signing_key.key,
        algorithms=['RS256'],
        audience=f'{settings.API_IDENTIFIER}'
    )


class DrfAuto0Authentication(authentication.BaseAuthentication):
    """ drf 自定义认证
        由于djangorestframework-jwt 不在维护，使用其他jwt框架也显得累赘。所以自定义一个drf 登录验证功能。
        auth0 官方范例，使用djangorestframework-jwt库，显得过时，而且累赘。
        原理：客户端(例如react)使用自己的方式登录auth0后，会得到access_token, 
              客户端通过设置 http authention 头，将access_token传递进来，
              本函数，以jwt的方式解释 access_token并进一步得到用户信息。
              
        
        settings 配置：
        REST_FRAMEWORK = {
            ...
            'DEFAULT_AUTHENTICATION_CLASSES':{
                ...
                mtlibs.auth0Auth.ExampleAuthentication
            }
        }
        
    """
    def authenticate(self, request):
        token = get_token_auth_header(request)
        
        decode_token = jwt_decode_token(token)
        username = jwt_get_username_from_payload_handler(decode_token)
        
        # 思考： 有没有必要将auto0的用户信息，映射到django 的用户信息？
        #        由或者，可以区分用户来自auth0 或者是django 内部用户。
        # if not username:
        #     return None

        try:
            user = User.objects.get(username=username)
            return (user, None) # 第二个 是auth 对象。
        except User.DoesNotExist:
            # 自动创建新用户，与之对应。
            new_user = User.objects.create(username="username",email="a@a.com",password="NJ87#hn@8xss6yd" )
            return (new_user, None) # 第二个 是auth 对象。
            # raise exceptions.AuthenticationFailed('No such user')


        # return (user, None) # 第二个 是auth 对象。
        


#!/usr/bin/env python3

##
## 运行当前项目默认构建任务
##
import os
import sys
import subprocess
from subprocess import run ,Popen
import shlex
from dotenv import load_dotenv, find_dotenv
from pathlib import Path
import argparse
import time
import logging
logging.basicConfig(level = logging.DEBUG,format = '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)

ENV_FILE = find_dotenv()
if ENV_FILE:
    load_dotenv(ENV_FILE)

load_dotenv(".env")

DOCKER_HUB_USER=os.environ.get("DOCKER_HUB_USER")
DOCKER_HUB_PASSWORD=os.environ.get("DOCKER_HUB_PASSWORD")
DOCKER_HUB_REGISTER=os.environ.get("DOCKER_HUB_REGISTER","")
IS_GITPOD = os.environ.get("USER") == "gitpod"

def login_dockerhub():
    if DOCKER_HUB_USER and DOCKER_HUB_PASSWORD:
        logger.info(f"登录dockerhub, 用户名: {DOCKER_HUB_USER}")
        cmd = f"echo {DOCKER_HUB_PASSWORD} | docker login {DOCKER_HUB_REGISTER} -u {DOCKER_HUB_USER} --password-stdin"
        logger.info(f"执行命令：{cmd}")
        run(cmd, check=True, shell=True)
    logger.info(f"没有指定docker hub 登录信息，跳过docker login")

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("urls", default=None, nargs="*", help="仓库地址, 如果指定了其他仓库，表示不是运行本项目的构建") 
    args = parser.parse_args()
    logger.info(f"urls: {args.urls}")

    urls = args.urls
    if not urls:
        # logger.info(f"没有输入urls参数，转为从环境变量中获取")
        # 从输入参数，或者环境变量中获取gitup网址。
        urls_from_env = os.environ.get("DOCKERBUILD_URL")
        if urls_from_env:
            urls = urls_from_env.split("|")

    else:
        logger.warn(f"远程构建暂时不支持")        
    #克隆到临时目录
    logger.info(f"[docker login]")
    login_dockerhub()
    run( shlex.split(f"pip3 install -U mtlibs"), check=True)
    if Path("./bin/build").exists():
        logger.info(f"docker build 使用脚本 ./bin/build")
        run(f"bin/build", check=True)

    # elif Path("./compose.yml"):
    #     logger.info(f"docker build 使用脚本: docker-compose build prebuild")
    #     run(shlex.split(f"docker-compose build prebuild"), check=True)
    #     run(shlex.split(f"docker-compose push prebuild"), check=True)
    else:
        logger.error(f"找不到 docker build 的相关脚本")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import os
import subprocess
from pathlib import Path
import subprocess
import logging
from mtxp import settings
from mtlibs import process_helper
from mtlibs import mtutils
import shutil
logger = logging.getLogger(__name__)

class NginxService():
    def __init__(self):
        self.html_root = settings.getHtmlRoot()
        logger.info(f"html root {self.html_root}")
        self.smapiRoot = "/mtxp"

    def start(self):
        logger.info("启动nginx 进程")
        
        if Path("/etc/nginx/conf.d/default.conf").exists():
            os.remove("/etc/nginx/conf.d/default.conf")
        port = 80
        check_port = mtutils.get_tcp_open_port(80)
        if not check_port:
            logger.info(f"tcp port opend {port},skip start nginx")
        else:
            setup_nginx(
                smApiPrefix=self.smapiRoot,
                htmlRoot=self.html_root,
                port=port,
                server_name="127.0.0.1"
            )

    def stop(self):
        pass


def setup_nginx(
    smApiPrefix: str,
    htmlRoot: str="/var/www/html",
    port: int = 80,
    server_name:str="127.0.0.1"
):
    nginx_conf = """user nginx;
worker_processes auto;

error_log /var/log/nginx/error.log notice;
pid /var/run/nginx.pid;

events {
  worker_connections 1024;
}

http {
  upstream mtxtun {
    server 127.0.0.1:3000 weight=99 max_fails=12 fail_timeout=60s;
  }
  upstream smirror {
    server 127.0.0.1:3456 weight=100 max_fails=12 fail_timeout=60s;
  }
  upstream mtx {
    server 127.0.0.1:8000 weight=200 max_fails=12 fail_timeout=60s;
  }
  upstream mtxp {
    server 127.0.0.1:5000 weight=300 max_fails=12 fail_timeout=60s;
  }
  upstream default_backend {
    server 127.0.0.1:3000 weight=450 max_fails=12 fail_timeout=60s;
  }

  include /etc/nginx/mime.types;
  default_type application/octet-stream;

  log_format main '$remote_addr - $remote_user [$time_local] "$request" '
  '$status $body_bytes_sent "$http_referer" '
  '"$http_user_agent" "$http_x_forwarded_for"';

  access_log /var/log/nginx/access.log main;

  sendfile on;
  #tcp_nopush     on;
  keepalive_timeout 65;
  gzip on;
  types_hash_max_size 2048;
  include /etc/nginx/conf.d/*.conf;
}
"""
    # print("nginx 配置信息", nginx_conf)
    nginx_default_site = """


server {
  listen @@PORT@@;
  server_name @@SERVER_NAME@@;
  #access_log  /var/log/nginx/host.access.log  main;
  # location / {
  #   root /app/static;
  #   index index.html index.htm;
  # }
  index index.php;
  # access_log /etc/nginx/conf.d/log/access.log;
  # error_log /etc/nginx/conf.d/log/error.log;
  # location / {
  #   add_header X-Powered-By 'PHP';
  #   root /var/www/html;
  #   index index.php index.html index.htm;
  #   # try_files @sm $uri $uri/;
  #   # try_files $uri /index.php?$args $uri/index.html $uri.html @default_backend;
  #   try_files $uri $uri/index.html $uri.html @default_backend;

  #   # # proxy_pass http://default_backend;
  #   # # autoindex on;
  #   # proxy_http_version 1.1;
  #   # index index.html index.htm;
  #   # proxy_set_header Upgrade $http_upgrade;
  #   # proxy_set_header Connection "Upgrade";
  #   # proxy_set_header Host $host;
  #   # proxy_set_header Host $host;
  #   # proxy_set_header X-Real-IP $remote_addr;
  #   # proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
  # }
  root @@STATIC_DIR@@;
  location / {
    autoindex on;
    index index.php index.html index.htm;
    try_files $uri $uri/ /index.php?$args @mtx default_backend;
    # try_files $uri $uri/ /index.php?$args;
    # proxy_pass http://mtx;
  }

  location ^~ /smirror {
    add_header X-Powered-By 'PHP';
    # root /app/static;
    # try_files @nextfront $uri $uri/;
    proxy_pass http://smirror;
    # autoindex on;
    # index index.html index.htm;
  }

  location ^~ /mtxcms/ {
    add_header X-Powered-By 'PHP';
    # try_files @nextfront $uri $uri/;
    proxy_pass http://default_backend;
  }


  location ^~ @@SM_API@@/ {
    add_header X-Powered-By 'PHP';
    # root /app/static;
    # try_files @nextfront $uri $uri/;
    proxy_pass http://mtxp;
    # autoindex on;
    # index index.html index.htm;
  }

  location ^~ /admin/ {
    add_header X-Powered-By 'PHP';
    # try_files @nextfront $uri $uri/;
    proxy_pass http://mtx;
    # autoindex on;
    # index index.html index.htm;
  }

  error_page 500 502 503 504 /50x.html;
  location = /50x.html {
    root /usr/share/nginx/html;
  }

  location /baidu {
    try_files /baidu.html
    $uri $uri/index.html $uri.html
    @fallback1;
  }
  #跳转到百度页面
  location @fallback {
    # rewrite ^/(.*)$ http://www.baidu.com;
    proxy_pass http://smirror;
  }

  location @default_backend {
    proxy_pass http://default_backend;
  }
  location @mtx {
    proxy_pass http://mtx;
  }


  # deny access to .htaccess files, if Apache's document root
  # concurs with nginx's one
  location ~ /\.ht {
    deny all;
  }
  # proxy the PHP scripts to Apache listening on 127.0.0.1:80
  #
  #location ~ \.php$ {
  #    proxy_pass   http://127.0.0.1;
  #}

  # pass the PHP scripts to FastCGI server listening on 127.0.0.1:9000
  #
  location ~ \.php$ {
    # root /var/www/html;
    # try_files $uri =404;
    fastcgi_pass 127.0.0.1:9000;
    # fastcgi_pass /run/php/php7.4-fpm.sock;
    # 设置nginx的默认首页文件(上面已经设置过了，可以删除)
    fastcgi_index index.php;
    fastcgi_param SCRIPT_FILENAME $document_root$fastcgi_script_name;
    include fastcgi_params;
  }
  location ~* \.(js|css|png|jpg|jpeg|gif|ico|svg)$ {
    expires max;
    log_not_found off;
  }
  # #ignored: “-” thing used or unknown variable in regex/rew
  # rewrite ^/([_0-9a-zA-Z-]+/)?wp-admin$ /$1wp-admin/ permanent;

  # if (-f $request_filename) {
  #   set $rule_2 1;
  # }
  # if (-d $request_filename) {
  #   set $rule_2 1;
  # }
  # if ($rule_2 = "1") {
  #   #ignored: “-” thing used or unknown variable in regex/rew
  # }
  # rewrite ^/([_0-9a-zA-Z-]+/)?(wp-(content|admin|includes).*) /$2 last;
  # rewrite ^/([_0-9a-zA-Z-]+/)?(.*.php)$ /$2 last;
  # rewrite /. /index.php last;

  # deny access to .htaccess files, if Apache's document root
  # concurs with nginx's one
  #
  location ~ /\.ht {
    deny all;
  }
  location ~* \.(xml|yaml|cmd|cfg|yml|tmp|sh|bat|txt|ts|tsx|jsx|lock)$ {
    # 禁用某些后缀名文件访问。
    deny all;
  }
  location ~ (README\.md|ockerfile.*|package.*\.json|.*config.js|.*prisma.*)$ {
    deny all;
  }
  location ^~ /(configs|build|log|logs)/ {
    #禁用某些目录访问
    deny all;
  }
  location ^~ /. {
    # 禁止以.开始的文件访问
    # 匹配任何以 /. 开头的地址，匹配符合以后，停止往下搜索正则，采用这一条。
    deny all;
  }
  location = /some2.html {
    rewrite ^/some2.html$ /test/2.html break;
  }
}


"""
    with open("/etc/nginx/nginx.conf", "w") as f:
        f.write(nginx_conf)

    if not Path(htmlRoot).exists():
        Path(htmlRoot).mkdir(parents=True, mode=0o777, exist_ok=True)

    with open("/etc/nginx/conf.d/default.conf", "w") as f:
        f.write(nginx_default_site\
                            .replace("@@PORT@@", str(port))\
                            .replace("@@SERVER_NAME@@", server_name)\
                            .replace("@@STATIC_DIR@@", htmlRoot)\
                            .replace("@@SM_API@@", smApiPrefix))

    cp: subprocess.CompletedProcess = process_helper.exec("nginx -t")
    if cp.returncode != 0:
        print("nginx 配置信息不正确")
        print(cp.stderr + cp.stdout)
        return
    nginx_cp = process_helper.exec("nginx -s reload", check=False)
    nginx_cp = process_helper.exec("nginx", check=False)
    if nginx_cp.returncode == 0:
        print("nginx 成功启动")




import ipaddress
import proxy
import asyncio
from csservices import portforward
async def on_command(args):
    """端口转发"""
    print('port====')
    #暂时写死
    options={
        "ports":[
                        "http://0.0.0.0:8080@www.google.com:80",
                        # "http://0.0.0.0:9090@127.0.0.1:49090"
            ]
        }
    loop  = asyncio.get_event_loop()
    task = loop.create_task(portforward.PortService().start(options))       
    # asyncio.run(task)
    done, pending = await asyncio.wait({task})
    print('命令运行结束')     
    # deamonTasks.append(task)


import requests
import json
import logging
logger = logging.getLogger(__name__)

def torIpCheck(proxyurl):
    """
        通过指定tor官方页面检测当前代理的ip情况。        
        检查代理服务是否有效，
        注意这里试用的是requests 的proxy url 方式的字符串。
        例如：  socks5h://mt:password@8.210.5.18:41080
                http://mt:password@8.210.5.18:8080
                https://abc.com
    """
    targetUrl='https://check.torproject.org/api/ip'
    logger.info(f"proxy check {proxyurl}, ".format(proxyurl=proxyurl))
    try:
        response = requests.get(targetUrl,
                                proxies={
                                    "http": proxyurl,
                                    "https": proxyurl,
                                },
                                verify=False)
        logger.debug(f"proxyCheck 响应: {response.content}")
        return response.json()
    
    except Exception as e:
        logger.exception(e)
    return None